/*
This object defines the JsListView Object. A JsListView is an object
that displays data both in a grid or a treeview, depending on how data
is structured.

It has a paginator for the grid display which makes it easier to navigate
throught data.

Also it has sortable columns on both grid and treeview displays.
The sort method can be disabled, for cases in which is not necessary
or desireable
*/
	/********************************************
				JsListView Methods
	********************************************/

	JsListView = function(name)
	{
		//listview properties
		this.name						= name;
		this.id							= "_" + this.name;
		this.type						= "JsListView";
		this.parent					= this;
		this.listview				= this;
		this.istreeview			= false;
		this.menu						= null;
		this.setopen				= true;
		this.sortcol	 			= 0;
		this.sortorder			= "updown";
		this.sortable				= true;
		this.pageable				= true;
		this.headers				= new Array();
		this.handlers				= new Array();
		this.childList			= new Array();
		this.width					= "100%";
		this.height					= "100%";
		this.selectedItem		= null;

		//JsObject events
		this.events				= new Array();

		//listview main container
		this.output 			= document.createElement("div");

		//listview header container
		this.lvheaderdiv		= document.createElement("div");
		this.lvheadertab 		= document.createElement("table");
		this.lvheaderrow 		= this.lvheadertab.insertRow(this.lvheadertab.rows.length);

		//listview body
		this.lvbodydiv 			= document.createElement("div");

		//styles
		this.output.className 		= "lv";
		this.output.style.width 	= this.width;
		this.output.style.height 	= this.height;
		this.lvheaderdiv.className 	= "lvheaderdiv";
		this.lvheadertab.className 	= "lvheadertab";
		this.lvheadertab.style.width	= 0;

		//Now we append them all together
		//lv header first
		this.output.appendChild(this.lvheaderdiv);
		this.lvheaderdiv.appendChild(this.lvheadertab);

		//then lv body
		this.output.appendChild(this.lvbodydiv);

		//scroll with fixed header
		this.setEvent("scroll",jslvScroll);	
	}

	/********************************************
				  Exclusive Methods
	********************************************/
	
	//empties the Listview itens
	JsListView.prototype.clearData = function()
	{
		for (i in this.childList)
		{
			this.lvbodydiv.removeChild(this.childList[i].output);
		}
		this.childList				= new Array();
		this.lvbodydiv.style.top	= 0;
		this.lvbodydiv.style.left	= 0;
		this.lvheaderdiv.style.top	= 0;
		this.lvheaderdiv.style.left	= 0;
	}

	//Add header for listview column
	JsListView.prototype.addColumn = function(headerStr, headerWidth, headerIcon)
	{
		var headerIndex = this.headers.length;
		this.headers[headerIndex] = this.lvheaderrow.insertCell(headerIndex * 2);
		this.handlers[headerIndex] = this.lvheaderrow.insertCell(headerIndex * 2 + 1);

		this.headers[headerIndex].style.width = headerWidth - 10;
		this.handlers[headerIndex].style.width = 10;
		
		//append img for handler
		handler_img = new JsLabel("handler_img");
		handler_img.setAttribute("objparent",this.name);
		handler_img.setAttribute("col",headerIndex);
		handler_img.setEvent("mousedown",beginResize);
		handler_img.setCursor("w-resize");
		handler_img.setWidth(10);
		handler_img.setHeight(20);
		handler_img.setValue("        ");
		handler_img.labeltab.className = "lvheaderhandler";
		handler_img.labelcell.className = "lvheaderhandler";
		if (handler_img.iconcell)
			handler_img.iconcell.className = "lvheaderhandler";

		//append text
		headertxt = new JsLabel("headertxt");
		headertxt.setValue(headerStr);
		headertxt.setHeight(22);
		headertxt.setIcon("images/down.gif","right");
		
		if(headerIcon)
			headertxt.setIcon(headerIcon);
		this.headers[headerIndex].appendChild(headertxt.output);
		
		this.handlers[headerIndex].appendChild(handler_img.output);
		
		//if listview is sortable, add method to the header
		if (this.sortable)
		{
			headertxt.setEvent("click", jslvSort);
			headertxt.setAttribute("col", headerIndex);
			headertxt.setAttribute("objparent", this.name);
		}

		this.headers[headerIndex].className = "lvheadercell";
		this.handlers[headerIndex].className = "lvheaderhandlercell";
		
		this.lvheadertab.style.width = parseInt(this.lvheadertab.style.width) + headerWidth;
	}

	//Sorts a column when header is clicked
	JsListView.prototype.sort = function(col)
	{
		alert(col);
	}
	
	JsListView.prototype.setTreeView = function()
	{
		this.istreeview	= true;
	}

	/********************************************
				  Exclusive Methods
	********************************************/

	/********************************************
				   Content Methods
	********************************************/

	//Adds a ListViewItem to the ListView children list
	//Only ListViewItens may be added to the ListView
	JsListView.prototype.addItem = function(obj)
	{
		var obj;
		if (obj.type != "JsListViewItem")
		{
			alert("Invalid object type for addition on listview.");
			return false;
		}
		
		js_alreadyexists = false;
		for (js_i in this.childList)
		{
			if (this.childList[js_i] == obj)
				js_alreadyexists = true
		}
	
		if (!js_alreadyexists)
			this.childList[this.childList.length] = obj;
			
		obj.parent = this;
		obj.listview = this;
		this.lvbodydiv.appendChild(obj.output);
	}

	//Deletes a ListViewItem from the ListView children list
	JsListView.prototype.delItem = function(obj)
	{
		var obj;
		var tempList = new Array();
		for (var js_i in this.childList)
		{
			if (this.childList[js_i].name != obj.name)
				tempList[tempList.length] = this.childList[js_i];
			else
			{
				obj.output.parentNode.removeChild(obj.output);
				delete obj;
			}
		}
		this.childList = new Array();
		this.childList = tempList;
		tempList = null;
	}

	//not implemented
	JsListView.prototype.setItem = function()
	{
	}

	/********************************************
				End of Content Methods
	********************************************/

	/********************************************
				   Values Methods
	********************************************/

	//Returns an array with the values of the children
	JsListView.prototype.getValue = function()
	{
		var values = new Array();
		for (var js_i in this.childList)
			values[js_i] = this.childList[js_i].getValue();

		return values;
	}

	//not implemented
	JsListView.prototype.setValue = function()
	{
	}

	JsListView.prototype.setAttribute = function(name,value)
	{
		eval("this." + name + " = \"" + value + "\"");
		eval("this.output." + name + " = \"" + value + "\"");
	}

	/********************************************
				End of Values Methods
	********************************************/

	/********************************************
				   Events Methods
	********************************************/

	JsListView.prototype.setMenu = function(obj)
	{
		this.output.menu = obj;
		if (browserType=="ie")
		{
			this.output.attachEvent("oncontextmenu",showContextMenu);
		}
		else
		{
			this.output.addEventListener("contextmenu",showContextMenu,false);
		}
	}

	JsListView.prototype.setEvent = function(eventStr,fnc)
	{
		eval("if (this.events['"+eventStr+"']){eventindex = this.events['"+eventStr+"'].length} else {this.events['"+eventStr+"'] = new Array(); eventindex = 0;}")
		eval("eventregistered = false;");
		eval("for (var js_i in this.events['"+eventStr+"']){if(this.events['"+eventStr+"'][js_i] == fnc){eventregistered=true;}}");
		eval("if(!eventregistered){ this.events['"+eventStr+"'][eventindex] = fnc;}");

		if (browserType == "ie")
		{
			this.output.attachEvent("on" + eventStr, fnc);
		}
		else
		{
			this.output.addEventListener(eventStr, fnc, false);
		}
	}

	JsListView.prototype.unsetEvent = function(eventStr,fnc)
	{
		eval("for (var js_i in this.events['"+eventStr+"']){if(this.events['"+eventStr+"'][js_i] == fnc){this.events['"+eventStr+"'].splice(js_i);}}");

		if (browserType == "ie")
		{
			this.output.detachEvent("on" + eventStr, fnc);
		}
		else
		{
			this.output.removeEventListener(eventStr, fnc, false);
		}
	}

	JsListView.prototype.disableEvent = function(eventStr,fnc)
	{
		if (browserType == "ie")
		{
			this.output.detachEvent("on" + eventStr, fnc);
		}
		else
		{
			this.output.removeEventListener(eventStr, fnc, false);
		}
	}

	/********************************************
				End of Events Methods
	********************************************/

	/********************************************
				   Visual Methods
	********************************************/

	JsListView.prototype.setHeight = function(value)
	{
		this.output.style.height = value
	}

	JsListView.prototype.setWidth = function(value)
	{
		this.output.style.width = value
	}

	JsListView.prototype.setClass = function(value)
	{
		this.output.className = value
	}

	JsListView.prototype.setCursor = function(value)
	{
		this.output.style.cursor = value
	}

	JsListView.prototype.setFont = function(value)
	{
		this.output.style.fontFamily = value
	}

	JsListView.prototype.setFontColor = function(value)
	{
		this.output.style.color = value
	}

	JsListView.prototype.setFontSize = function(value)
	{
		this.output.style.fontSize = value
	}

	JsListView.prototype.setFontWeight = function(value)
	{
		this.output.style.fontWeight = value
	}

	JsListView.prototype.setFontItalic = function(value)
	{
		this.output.style.fontStyle = value
	}

	/********************************************
				End of Visual Methods
	********************************************/

	/********************************************
				  General Methods
	********************************************/

	JsListView.prototype.disable = function(value)
	{
		this.combo.disabled = value;
		if (value)
		{
			for (var js_i in this.events)
			{
				for (var js_j in this.events[js_i])
					this.disableEvent([js_i],this.events[js_i][js_j])
			}
		}
		else
		{
			for (var js_i in this.events)
			{
				for (var js_j in this.events[js_i])
					this.setEvent(js_i,this.events[js_i][js_j])
			}
		}
	}

	/********************************************
			  End of General Methods
	********************************************/

	function jslvScroll(e)
	{
		if (browserType == "ie")
		{
  			e = window.event;
			e.returnValue = false;

			obj = e.srcElement;
			obj.childNodes[0].style.top = obj.scrollTop;
		}
		else
		{
			e.preventDefault();
			this.childNodes[0].style.top = this.scrollTop;
		}
	}
	
	function uplsvsort(a,b)
	{
		if (a.cells[js_scol].getValue() > b.cells[js_scol].getValue())
			return 1;
		else
			return -1;
		
		return 0;
	}
	
	function downlsvsort(a,b)
	{
		if (a.cells[js_scol].getValue() < b.cells[js_scol].getValue())
			return 1;
		else
			return -1;
		
		return 0;
	}
	
	function jslvSort(e)
	{
		if (browserType == "ie")
		{
			e = window.event;
			obj = e.srcElement;
		}
		else
		{
			obj = e.target;
		}
		js_scol = obj.col;
		var jslvobj = eval(obj.objparent);
		for (js_i in jslvobj.headers)
			jslvobj.headers[js_i].childNodes[0].childNodes[0].childNodes[0].childNodes[0].childNodes[1].childNodes[0].src="images/down.gif";
		mark = jslvobj.headers[js_scol].childNodes[0].childNodes[0].childNodes[0].childNodes[0].childNodes[1].childNodes[0];
		
		jslvOrder(jslvobj,js_scol);	
		
	}
	
	function jslvOrder(obj,col)
	{
		var obj;
		var col;
		
		if (col == obj.sortcol)
		{
			if (obj.sortorder == "updown")
			{
				func = uplsvsort;
				obj.sortorder = "downup";
				mark.src = "images/down_arrow.gif";
			}
			else
			{
				func = downlsvsort;
				obj.sortorder = "updown";
				mark.src = "images/up_arrow.gif";
			}
		}
		else 
		{
			var func = uplsvsort;
			obj.sortorder = "downup";
			obj.sortcol = js_scol;
			mark.src = "images/down_arrow.gif";
		}
		
		obj.childList.sort(func);
		
		for (js_i in obj.childList)
		{
			if (obj.type == "JsListViewItem")
			{
				obj.lvitemchilddiv.appendChild(obj.childList[js_i].output);
			}
			else
				obj.output.appendChild(obj.childList[js_i].output);
			if (obj.childList[js_i].childList.length > 0)
				jslvOrder(obj.childList[js_i],col)
		}
	}
	
	//Variaveis globais acessiveis pelas funcoes de resize das colunas
   var beginXpos;
   var endXpos;
   var colIndex = null;
   var lstView = null;
	
	function beginResize(e)
	{
		if (!e) var e = window.event;
		if (e.clientX)
			beginXpos = (e.clientX + document.body.scrollLeft);
		else
			beginXpos = (e.pageX + document.body.scrollLeft);
		
		if (e.target)
		{
			lstView = eval(e.target.objparent);
			colIndex = e.target.col;
		}
		else
		{
			lstView = eval(e.srcElement.objparent);
			colIndex = e.srcElement.col;
		}
		
		document.onmousemove = resizeMove;
		document.onmouseup   = endResize;
		
		return false;
	}

	function resizeMove(e)
	{
		if (!e) var e = window.event;
		
		if (lstView)
		{
			if (e.clientX)
				endXpos = (e.clientX + document.body.scrollLeft);
			else
				endXpos = (e.pageX + document.body.scrollLeft);
			
			size = parseInt(endXpos) - parseInt(beginXpos);
			
			col_width = parseInt(lstView.headers[colIndex].style.width);
			list_width = parseInt(lstView.lvheadertab.style.width);
			
			if ((col_width + size)>0)
			{
				lstView.headers[colIndex].style.width = (col_width + size);
				lstView.lvheadertab.style.width = (list_width + size);
			}
		
		beginXpos = endXpos;
		return false;
		}
	}
	
	function setAllColSizes(obj)
	{
		if (browserType=="ie")
			var fixsize = 10;
		else
			var fixsize = 12;
		
		var obj;
		var js_i;
		
		for (js_i in obj.childList)
		{
			obj.childList[js_i].lvitemrow.childNodes[colIndex].style.width = parseInt(obj.listview.headers[colIndex].style.width) + fixsize;
			obj.childList[js_i].lvitemtab.style.width = parseInt(obj.listview.lvheadertab.style.width);
			if (obj.childList[js_i].childList.length > 0)
				setAllColSizes(obj.childList[js_i]);
		}
	}
	
	function endResize(e)
	{
		setAllColSizes(lstView)
		colIndex = null;
		lstView = null;
		document.onmousemove = null;
		document.onmouseup   = null;
		disableSelection();
		return false;
	}
